package com.streamlinecontrol.scichartjava;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import com.sun.jna.Native;
import com.sun.jna.Platform;
import com.sun.jna.Pointer;

public class App {
    static {
        // print os convert number to enum
        int os = Platform.getOSType();
        if (os == 1) {
            System.out.println("OS: Linux");
        } else if (os == 2) {
            System.out.println("OS: Windows");
        } else if (os == 3) {
            System.out.println("OS: Mac");
        } else if (os == 4) {
            System.out.println("OS: Solaris");
        } else if (os == 5) {
            System.out.println("OS: AIX");
        } else {
            System.out.println("OS: Unknown");
        }

        System.out.println("Arch: " + Platform.ARCH);
        // TODO: will need to be adjusted for architecture
        String libSciChartLicenseServer = "bin/linux-x64/native/libSciChartLicenseServer.so"; // Adjust for your library name and
        String libSciChartSCS = "bin/linux-x64/native/libSciChartSCS.so"; // Adjust for your library name and
        String[] libNames = {libSciChartLicenseServer, libSciChartSCS};

        try {
            // Step 1: Use Files.createTempDirectory to create a temporary directory
            Path tempDir = Files.createTempDirectory("native-lib");

            // Step 2: Check if a directory with the desired name already exists
            Path desiredDir = Paths.get(tempDir.getParent().toString(), "native-lib");
            if (Files.exists(desiredDir)) {
                // Step 3: Delete the existing directory
                Files.walk(desiredDir)
                     .map(Path::toFile)
                     .sorted((o1, o2) -> -o1.compareTo(o2))
                     .forEach(File::delete);
            }

            // Step 4: Create the directory with the desired name and delete the temporary directory
            tempDir.toFile().delete();
            Files.createDirectories(desiredDir);

            // load all native libraries
            for (String name : libNames) {
                // Load the native library from the JAR
                InputStream in = App.class.getResourceAsStream("/" + name);
                if (in != null) {
                    // Create a temporary directory
                    // Copy the native library from the JAR to the temporary directory
                    Path tempLibPath = desiredDir.resolve(name.substring(name.lastIndexOf('/') + 1));
                    Files.copy(in, tempLibPath);
                    // Load the native library
                    System.load(tempLibPath.toAbsolutePath().toString());

                    System.out.println("Library " + name + " loaded");
                }
                in.close();
            }

        } catch (Exception e) {
            throw new RuntimeException("Failed to load native library", e);
        }
    }

    public static void main(String[] args) {
        try {
        // set the license key
        String licenseKey = "yourLicenseKeyHere";
        int result = SciChartSCS.SciChartSCSLibrary.INSTANCE.SciChartSCS_SetRuntimeLicenseKey(licenseKey);
        System.out.println("Result: " + result);

        Pointer pointer = SciChartSCS.SciChartSCSLibrary.INSTANCE.SciChartSCS_GetLicenseErrors();
        String errors = pointer.getString(0, StandardCharsets.UTF_8.name());
        System.out.println("Errors: " + errors);
        Native.free(Pointer.nativeValue(pointer));

        Pointer validation = SciChartSCS.SciChartSCSLibrary.INSTANCE.SciChartSCS_ValidateChallenge("yourChallengeHere");
        System.out.println("Validation: " + validation.getString(0, StandardCharsets.UTF_8.name()));
        Native.free(Pointer.nativeValue(validation));

        // int result = SciChartLicenseServer.SciChartLibrary.INSTANCE.SciChartLicenseServer_SetRuntimeLicenseKey(licenseKey);
        // System.out.println("Result: " + result);
        //
        // Pointer pointer = SciChartLicenseServer.SciChartLibrary.INSTANCE.SciChartLicenseServer_GetLicenseErrors();
        // String errors = pointer.getString(0, StandardCharsets.UTF_8.name());
        // System.out.println("Errors: " + errors);

        // byte[] bytes = pointer.getByteArray(0, 3096);
        // System.out.println("License: " + new String(bytes, StandardCharsets.UTF_8));
        // Native.free(Pointer.nativeValue(pointer));

        // set the assembly name
        // String assemblyName = "yourAssemblyNameHere";
        // SciChartLicenseServer.SciChartLibrary.INSTANCE.SciChartLicenseServer_SetAssemblyName(assemblyName);
        //
        // // Set the license key
        // String licenseKey = "yourLicenseKeyHere";
        // int result = SciChartLicenseServer.SciChartLibrary.INSTANCE
        //         .SciChartLicenseServer_SetRuntimeLicenseKey(licenseKey);
        // System.out.println("Result: " + result);
        //
        // // Get the license errors
        // Pointer errors = SciChartLicenseServer.SciChartLibrary.INSTANCE.SciChartLicenseServer_GetLicenseErrors();
        // System.out.println("Errors: " + errors.getString(1232, StandardCharsets.UTF_8.name()));
        //
        // // Validate the challenge
        // String challenge = "yourChallengeHere";
        // Pointer validation = SciChartLicenseServer.SciChartLibrary.INSTANCE
        //         .SciChartLicenseServer_ValidateChallenge(challenge);
        // System.out.println("Validation: " + validation.getString(1232, StandardCharsets.UTF_8.name()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
